//==- CodingConventionsChecker.h - Checker for coding rules -----*- C++ -*-==//
//
//                     The LLVM Compiler Infrastructure
//
// This file is distributed under the University of Illinois Open Source
// License. See LICENSE.TXT for details.
//
//===----------------------------------------------------------------------===//
//
//  This file defines the interface to a set of coding convention checkers.
//
//===----------------------------------------------------------------------===//

#ifndef LLVM_CLANG_ANALYSIS_CODINGCONVENTIONCHECKER_H
#define LLVM_CLANG_ANALYSIS_CODINGCONVENTIONCHECKER_H

namespace clang {

class Decl;
class Diagnostic;
class SourceManager;
class LangOptions;
class CodeModificationHint;
class SourceLocation;
class SourceRange;

class CodingConventionConfigReader {
public:
  virtual ~CodingConventionConfigReader();
  virtual bool processConfigLine(llvm::StringRef Line, size_t LineNo) = 0;
  virtual void endOfConfig() {}
};

class CodingConventionChecker {
  Diagnostic& Diags;
  SourceManager& SM;
  const LangOptions& Lang;

  static std::string ConfigFileName;

public:

  static void setConfigFile(llvm::StringRef filename);

  CodingConventionChecker(Diagnostic& diags, SourceManager& sm,
                           const LangOptions& lang):
    Diags(diags), SM(sm), Lang(lang) {}

  const Diagnostic& getDiagnostic() const { return Diags; }
  Diagnostic& getDiagnostic() { return Diags; }
  const SourceManager& getSourceManager() const { return SM; }
  SourceManager& getSourceManager() { return SM; }
  const LangOptions& getLangOptions() const { return Lang; }

  bool readConfig(llvm::StringRef Section, CodingConventionConfigReader& CR);
  void reportConfigError(llvm::StringRef Str, size_t LineNo);

  template <typename T>
  void report(T *D, llvm::StringRef Str, const CodeModificationHint& Fixit) {
    report(D->getLocStart(), D->getSourceRange(), Str, Fixit);
  }

  void report(const SourceLocation& Start, const SourceRange& Range,
              llvm::StringRef Str, const CodeModificationHint& Fixit);
};

} // end namespace clang

#endif
